@@ -21,9 +21,9 @@ from .lib import yellow, scatter, sip, toiter
 from .aws_queue_api import AWSTaskQueueAPI, AWS_BATCH_SIZE
 from .file_queue_api import FileQueueAPI
 from .paths import extract_path, mkpath
-from .registered_task import RegisteredTask, deserialize, totask, totaskid
 from .scheduler import schedule_jobs
-
+from .queueables import totask, totaskid
+from .queueablefns import FunctionTask
 
 def totalfn(iterator, total):
   if total is not None:
@@ -147,7 +147,11 @@ class TaskQueue(object):
     """
     return [ totask(x) for x in iter(self.api) ]
 
-  def insert(self, tasks, delay_seconds=0, total=None, parallel=1):
+  def insert(
+    self, tasks, delay_seconds=0, 
+    total=None, parallel=1, skip_insert_counter=False
+  ):
+    """Inserts tasks and returns number inserted."""
     if isinstance(tasks, TaskQueue):
       taskgen = tasks.tasks()
       if not isinstance(taskgen, TaskQueue):
@@ -159,8 +163,7 @@ class TaskQueue(object):
     total = totalfn(tasks, total)
 
     if parallel not in (1, False) and total is not None and total > 1:
-      multiprocess_upload(self.__class__, mkpath(self.path), tasks, parallel=parallel)
-      return
+      return multiprocess_upload(self.__class__, mkpath(self.path), tasks, parallel=parallel)
 
     try:
       batch_size = self.api.batch_size
@@ -169,7 +172,7 @@ class TaskQueue(object):
     
     bodies = (
       {
-        "payload": task.payload(),
+        "payload": totask(task).payload(),
         "queueName": self.path.path,
       } 
       for task in tasks
@@ -197,7 +200,12 @@ class TaskQueue(object):
       batch_size=batch_size,
     )
 
-    self.api.add_insert_count(ct[0])
+    if not skip_insert_counter:
+      self.api.add_insert_count(ct[0])
+    return ct[0]
+
+  def add_insert_count(self, ct):
+    self.api.add_insert_count(ct)
 
   def insert_all(self, *args, **kwargs):
     """For backwards compatibility."""
@@ -428,19 +436,29 @@ class LocalTaskQueue(object):
       delay_seconds=0, total=None,
       parallel=None, progress=True
     ):
+    tasks = toiter(tasks)
+    ct = 0
     for task in tasks:
       args, kwargs = [], {}
       if isinstance(task, tuple):
         task, args, kwargs = task
+      task = totask(task)
       task = {
         'payload': task.payload(),
         'id': -1,
       }
       self.queue.append( (task, args, kwargs) )
+      ct += 1
+
+    return ct
 
   def insert_all(self, *args, **kwargs):
-    self.insert(*args, **kwargs)
+    ct = self.insert(*args, **kwargs)
     self.execute(self.progress)
+    return ct
+
+  def add_insert_count(self, ct):
+    pass
 
   def poll(self, *args, **kwargs):
     pass
@@ -496,7 +514,7 @@ def _scatter(sequence, n):
 
 def soloprocess_upload(QueueClass, queue_name, tasks):
   tq = QueueClass(queue_name)
-  tq.insert(tasks)
+  return tq.insert(tasks, skip_insert_counter=True)
 
 error_queue = mp.Queue()
 
@@ -507,15 +525,15 @@ def multiprocess_upload(QueueClass, queue_name, tasks, parallel=True):
     raise ValueError("Parallel must be a positive number or zero (all cpus). Got: " + str(parallel))
 
   if parallel == 1:
-    soloprocess_upload(QueueClass, queue_name, tasks)
-    return 
-
+    return soloprocess_upload(QueueClass, queue_name, tasks)
+    
   def capturing_soloprocess_upload(*args, **kwargs):
     try:
-      soloprocess_upload(*args, **kwargs)
+      return soloprocess_upload(*args, **kwargs)
     except Exception as err:
       print(err)
       error_queue.put(err)
+    return 0
 
   uploadfn = partial(
     capturing_soloprocess_upload, QueueClass, queue_name
@@ -532,13 +550,16 @@ def multiprocess_upload(QueueClass, queue_name, tasks, parallel=True):
   try:
     task = next(item for item in tasks if item is not None)
   except StopIteration:
-    return 
+    return 0
 
   cls_module = task.__class__.__module__
   task.__class__.__module__ = '__main__'
 
   with pathos.pools.ProcessPool(parallel) as pool:
-    pool.map(uploadfn, tasks)
+    results = pool.map(uploadfn, tasks)
+
+  ct = sum(results)
+  QueueClass(queue_name).add_insert_count(ct)
 
   task.__class__.__module__ = cls_module
 
@@ -551,5 +572,7 @@ def multiprocess_upload(QueueClass, queue_name, tasks, parallel=True):
     if len(errors):
       raise Exception(errors)
 
+  return ct
+
 
 
